pub struct AbstractComprehension {
pub return_expr: Expression,
pub qualifiers: Vec<Qualifier>,
pub return_expr_symbols: Rc<RefCell<SymbolTable>>,
pub generator_symbols: Rc<RefCell<SymbolTable>>,
}Fields
return_expr: Expressionqualifiers: Vec<Qualifier>return_expr_symbols: Rc<RefCell<SymbolTable>>The symbol table used in the return expression.
Variables from generator expressions are “given” in the context of the return expression. That is, they are constants which are different for each expansion of the comprehension.
generator_symbols: Rc<RefCell<SymbolTable>>The scope for variables in generator expressions.
Variables declared in generator expressions are decision variables, since they do not have a constant value.
Implementations