pub struct ExpressionGenerator {
pub decl: DeclarationPtr,
pub expression: Expression,
}Fields
decl: DeclarationPtrexpression: ExpressionTrait Implementations
Sourceimpl Biplate<Expression> for ExpressionGenerator
impl Biplate<Expression> for ExpressionGenerator
Sourcefn biplate(
&self,
) -> (Tree<Expression>, Box<dyn Fn(Tree<Expression>) -> ExpressionGenerator>)
fn biplate( &self, ) -> (Tree<Expression>, Box<dyn Fn(Tree<Expression>) -> ExpressionGenerator>)
Definition of a Biplate. Read more
fn with_children_bi(&self, children: VecDeque<To>) -> Self
fn with_children_bi(&self, children: VecDeque<To>) -> Self
Reconstructs the node with the given children. Read more
fn descend_bi(&self, op: &impl Fn(To) -> To) -> Self
fn descend_bi(&self, op: &impl Fn(To) -> To) -> Self
Biplate variant of [
Uniplate::descend] Read morefn universe_bi(&self) -> VecDeque<To>
fn universe_bi(&self) -> VecDeque<To>
Gets all children of a node, including itself and all children. Read more
fn children_bi(&self) -> VecDeque<To>
fn children_bi(&self) -> VecDeque<To>
Returns the children of a type. If to == from then it returns the original element (in contrast to children). Read more
fn transform_bi(&self, op: &impl Fn(To) -> To) -> Self
fn transform_bi(&self, op: &impl Fn(To) -> To) -> Self
Applies the given function to all nodes bottom up. Read more
fn holes_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
fn holes_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
Returns an iterator over all direct children of the input, paired with a function that
“fills the hole” where the child was with a new value. Read more
fn contexts_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
fn contexts_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
Returns an iterator over the universe of the input, paired with a function that “fills the
hole” where the child was with a new value. Read more
Sourceimpl Biplate<ExpressionGenerator> for ExpressionGenerator
impl Biplate<ExpressionGenerator> for ExpressionGenerator
Sourcefn biplate(
&self,
) -> (Tree<ExpressionGenerator>, Box<dyn Fn(Tree<ExpressionGenerator>) -> ExpressionGenerator>)
fn biplate( &self, ) -> (Tree<ExpressionGenerator>, Box<dyn Fn(Tree<ExpressionGenerator>) -> ExpressionGenerator>)
Definition of a Biplate. Read more
fn with_children_bi(&self, children: VecDeque<To>) -> Self
fn with_children_bi(&self, children: VecDeque<To>) -> Self
Reconstructs the node with the given children. Read more
fn descend_bi(&self, op: &impl Fn(To) -> To) -> Self
fn descend_bi(&self, op: &impl Fn(To) -> To) -> Self
Biplate variant of [
Uniplate::descend] Read morefn universe_bi(&self) -> VecDeque<To>
fn universe_bi(&self) -> VecDeque<To>
Gets all children of a node, including itself and all children. Read more
fn children_bi(&self) -> VecDeque<To>
fn children_bi(&self) -> VecDeque<To>
Returns the children of a type. If to == from then it returns the original element (in contrast to children). Read more
fn transform_bi(&self, op: &impl Fn(To) -> To) -> Self
fn transform_bi(&self, op: &impl Fn(To) -> To) -> Self
Applies the given function to all nodes bottom up. Read more
fn holes_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
fn holes_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
Returns an iterator over all direct children of the input, paired with a function that
“fills the hole” where the child was with a new value. Read more
fn contexts_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
fn contexts_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
Returns an iterator over the universe of the input, paired with a function that “fills the
hole” where the child was with a new value. Read more
Sourceimpl Biplate<SubModel> for ExpressionGenerator
impl Biplate<SubModel> for ExpressionGenerator
Sourcefn biplate(
&self,
) -> (Tree<SubModel>, Box<dyn Fn(Tree<SubModel>) -> ExpressionGenerator>)
fn biplate( &self, ) -> (Tree<SubModel>, Box<dyn Fn(Tree<SubModel>) -> ExpressionGenerator>)
Definition of a Biplate. Read more
fn with_children_bi(&self, children: VecDeque<To>) -> Self
fn with_children_bi(&self, children: VecDeque<To>) -> Self
Reconstructs the node with the given children. Read more
fn descend_bi(&self, op: &impl Fn(To) -> To) -> Self
fn descend_bi(&self, op: &impl Fn(To) -> To) -> Self
Biplate variant of [
Uniplate::descend] Read morefn universe_bi(&self) -> VecDeque<To>
fn universe_bi(&self) -> VecDeque<To>
Gets all children of a node, including itself and all children. Read more
fn children_bi(&self) -> VecDeque<To>
fn children_bi(&self) -> VecDeque<To>
Returns the children of a type. If to == from then it returns the original element (in contrast to children). Read more
fn transform_bi(&self, op: &impl Fn(To) -> To) -> Self
fn transform_bi(&self, op: &impl Fn(To) -> To) -> Self
Applies the given function to all nodes bottom up. Read more
fn holes_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
fn holes_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
Returns an iterator over all direct children of the input, paired with a function that
“fills the hole” where the child was with a new value. Read more
fn contexts_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
fn contexts_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
Returns an iterator over the universe of the input, paired with a function that “fills the
hole” where the child was with a new value. Read more
Sourceimpl Clone for ExpressionGenerator
impl Clone for ExpressionGenerator
Sourcefn clone(&self) -> ExpressionGenerator
fn clone(&self) -> ExpressionGenerator
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for ExpressionGenerator
impl Debug for ExpressionGenerator
Sourceimpl<'de> Deserialize<'de> for ExpressionGenerator
impl<'de> Deserialize<'de> for ExpressionGenerator
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl Hash for ExpressionGenerator
impl Hash for ExpressionGenerator
Sourceimpl PartialEq for ExpressionGenerator
impl PartialEq for ExpressionGenerator
Sourceimpl Serialize for ExpressionGenerator
impl Serialize for ExpressionGenerator
Sourceimpl Uniplate for ExpressionGenerator
impl Uniplate for ExpressionGenerator
Sourcefn uniplate(
&self,
) -> (Tree<ExpressionGenerator>, Box<dyn Fn(Tree<ExpressionGenerator>) -> ExpressionGenerator>)
fn uniplate( &self, ) -> (Tree<ExpressionGenerator>, Box<dyn Fn(Tree<ExpressionGenerator>) -> ExpressionGenerator>)
Definition of a
Uniplate. Read morefn descend(&self, op: &impl Fn(Self) -> Self) -> Self
fn descend(&self, op: &impl Fn(Self) -> Self) -> Self
Applies a function to all direct children of this Read more
fn universe(&self) -> VecDeque<Self>
fn universe(&self) -> VecDeque<Self>
Gets all children of a node, including itself and all children. Read more
fn with_children(&self, children: VecDeque<Self>) -> Self
fn with_children(&self, children: VecDeque<Self>) -> Self
Reconstructs the node with the given children. Read more
fn transform(&self, f: &impl Fn(Self) -> Self) -> Self
fn transform(&self, f: &impl Fn(Self) -> Self) -> Self
Applies the given function to all nodes bottom up.
fn rewrite(&self, f: &impl Fn(Self) -> Option<Self>) -> Self
fn rewrite(&self, f: &impl Fn(Self) -> Option<Self>) -> Self
Rewrites by applying a rule everywhere it can.
fn cata<T>(&self, op: &impl Fn(Self, VecDeque<T>) -> T) -> T
fn cata<T>(&self, op: &impl Fn(Self, VecDeque<T>) -> T) -> T
Performs a fold-like computation on each value. Read more
impl Eq for ExpressionGenerator
impl StructuralPartialEq for ExpressionGenerator