pub fn unflatten_matrix<T: MatrixValue>(
elems: &[T],
index_domains: &[T::Dom],
strides: &[usize],
) -> TExpand description
“Un-flatten” a slice of elements into a Matrix with the given index domains
pub fn unflatten_matrix<T: MatrixValue>(
elems: &[T],
index_domains: &[T::Dom],
strides: &[usize],
) -> T“Un-flatten” a slice of elements into a Matrix with the given index domains