AsId

Struct AsId 

pub struct AsId;
Serialises the object’s ID. The actual data is NOT stored.

WARNING

The object is de-serialised with the correct ID, but an empty default value.

After de-serialising an entire structure, it is the user’s responsibility to find a complete copy of the object and restore the shared pointer to it. This should be possible as long as at least one instance of the object has been serialised with PtrAsInner.

See the de-serialisation code for [Model] for an example.