pub struct FieldEntryGround {
pub name: Name,
pub domain: Moo<GroundDomain>,
}Fields
name: Namedomain: Moo<GroundDomain>Trait Implementations
Sourceimpl Biplate<FieldEntryGround> for FieldEntryGround
impl Biplate<FieldEntryGround> for FieldEntryGround
Sourcefn biplate(
&self,
) -> (Tree<FieldEntryGround>, Box<dyn Fn(Tree<FieldEntryGround>) -> FieldEntryGround>)
fn biplate( &self, ) -> (Tree<FieldEntryGround>, Box<dyn Fn(Tree<FieldEntryGround>) -> FieldEntryGround>)
Definition of a Biplate. Read more
fn with_children_bi(&self, children: VecDeque<To>) -> Self
fn with_children_bi(&self, children: VecDeque<To>) -> Self
Reconstructs the node with the given children. Read more
fn descend_bi(&self, op: &impl Fn(To) -> To) -> Self
fn descend_bi(&self, op: &impl Fn(To) -> To) -> Self
Biplate variant of [
Uniplate::descend] Read morefn universe_bi(&self) -> VecDeque<To>
fn universe_bi(&self) -> VecDeque<To>
Gets all children of a node, including itself and all children. Read more
fn children_bi(&self) -> VecDeque<To>
fn children_bi(&self) -> VecDeque<To>
Returns the children of a type. If to == from then it returns the original element (in contrast to children). Read more
fn transform_bi(&self, op: &impl Fn(To) -> To) -> Self
fn transform_bi(&self, op: &impl Fn(To) -> To) -> Self
Applies the given function to all nodes bottom up. Read more
fn holes_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
fn holes_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
Returns an iterator over all direct children of the input, paired with a function that
“fills the hole” where the child was with a new value. Read more
fn contexts_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
fn contexts_bi(&self) -> impl Iterator<Item = (To, impl Fn(To))>
Returns an iterator over the universe of the input, paired with a function that “fills the
hole” where the child was with a new value. Read more
Sourceimpl Clone for FieldEntryGround
impl Clone for FieldEntryGround
Sourcefn clone(&self) -> FieldEntryGround
fn clone(&self) -> FieldEntryGround
Returns a duplicate of the value. Read more
1.0.0 (const: unstable) · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for FieldEntryGround
impl Debug for FieldEntryGround
Sourceimpl<'de> Deserialize<'de> for FieldEntryGround
impl<'de> Deserialize<'de> for FieldEntryGround
Sourcefn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(__deserializer: __D) -> Result<Self, __D::Error>where
__D: Deserializer<'de>,
Deserialize this value from the given Serde deserializer. Read more
Sourceimpl From<FieldEntryGround> for FieldEntry
impl From<FieldEntryGround> for FieldEntry
Sourcefn from(value: FieldEntryGround) -> Self
fn from(value: FieldEntryGround) -> Self
Converts to this type from the input type.
Sourceimpl Hash for FieldEntryGround
impl Hash for FieldEntryGround
Sourceimpl PartialEq for FieldEntryGround
impl PartialEq for FieldEntryGround
Sourcefn eq(&self, other: &FieldEntryGround) -> bool
fn eq(&self, other: &FieldEntryGround) -> bool
Tests for
self and other values to be equal, and is used by ==.Sourceimpl Quine for FieldEntryGround
impl Quine for FieldEntryGround
fn ctor_tokens(&self) -> TokenStream
Sourceimpl Serialize for FieldEntryGround
impl Serialize for FieldEntryGround
Sourceimpl TryFrom<FieldEntry> for FieldEntryGround
impl TryFrom<FieldEntry> for FieldEntryGround
Sourcetype Error = DomainOpError
type Error = DomainOpError
The type returned in the event of a conversion error.
Sourceimpl Uniplate for FieldEntryGround
impl Uniplate for FieldEntryGround
Sourcefn uniplate(
&self,
) -> (Tree<FieldEntryGround>, Box<dyn Fn(Tree<FieldEntryGround>) -> FieldEntryGround>)
fn uniplate( &self, ) -> (Tree<FieldEntryGround>, Box<dyn Fn(Tree<FieldEntryGround>) -> FieldEntryGround>)
Definition of a
Uniplate. Read morefn descend(&self, op: &impl Fn(Self) -> Self) -> Self
fn descend(&self, op: &impl Fn(Self) -> Self) -> Self
Applies a function to all direct children of this Read more
fn universe(&self) -> VecDeque<Self>
fn universe(&self) -> VecDeque<Self>
Gets all children of a node, including itself and all children. Read more
fn with_children(&self, children: VecDeque<Self>) -> Self
fn with_children(&self, children: VecDeque<Self>) -> Self
Reconstructs the node with the given children. Read more
fn transform(&self, f: &impl Fn(Self) -> Self) -> Self
fn transform(&self, f: &impl Fn(Self) -> Self) -> Self
Applies the given function to all nodes bottom up.
fn rewrite(&self, f: &impl Fn(Self) -> Option<Self>) -> Self
fn rewrite(&self, f: &impl Fn(Self) -> Option<Self>) -> Self
Rewrites by applying a rule everywhere it can.
fn cata<T>(&self, op: &impl Fn(Self, VecDeque<T>) -> T) -> T
fn cata<T>(&self, op: &impl Fn(Self, VecDeque<T>) -> T) -> T
Performs a fold-like computation on each value. Read more
impl Eq for FieldEntryGround
impl StructuralPartialEq for FieldEntryGround