pub struct Model {
pub search_order: Option<Vec<Name>>,
pub dominance: Option<Expression>,
pub context: Arc<RwLock<Context<'static>>>,
/* private fields */
}Expand description
An Essence model.
Fields
search_order: Option<Vec<Name>>dominance: Option<Expression>context: Arc<RwLock<Context<'static>>>Implementations
Sourceimpl Model
impl Model
Sourcepub fn new(context: Arc<RwLock<Context<'static>>>) -> Model
pub fn new(context: Arc<RwLock<Context<'static>>>) -> Model
Creates a new top-level model from the given context.
Sourcepub fn new_in_parent_scope(parent: SymbolTablePtr) -> Model
pub fn new_in_parent_scope(parent: SymbolTablePtr) -> Model
Creates a new model whose symbol table has parent as parent scope.
Sourcepub fn symbols_ptr_unchecked(&self) -> &SymbolTablePtr
pub fn symbols_ptr_unchecked(&self) -> &SymbolTablePtr
The symbol table for this model as a pointer.
Sourcepub fn symbols_ptr_unchecked_mut(&mut self) -> &mut SymbolTablePtr
pub fn symbols_ptr_unchecked_mut(&mut self) -> &mut SymbolTablePtr
The symbol table for this model as a mutable pointer.
Sourcepub fn symbols(&self) -> RwLockReadGuard<'_, SymbolTable>
pub fn symbols(&self) -> RwLockReadGuard<'_, SymbolTable>
The symbol table for this model as a reference.
Sourcepub fn symbols_mut(&mut self) -> RwLockWriteGuard<'_, SymbolTable>
pub fn symbols_mut(&mut self) -> RwLockWriteGuard<'_, SymbolTable>
The symbol table for this model as a mutable reference.
Sourcepub fn root(&self) -> &Expression
pub fn root(&self) -> &Expression
The root node of this model.
Sourcepub fn root_mut_unchecked(&mut self) -> &mut Expression
pub fn root_mut_unchecked(&mut self) -> &mut Expression
The root node of this model, as a mutable reference.
The caller is responsible for ensuring that the root node remains an Expression::Root.
Sourcepub fn replace_root(&mut self, new_root: Expression) -> Expression
pub fn replace_root(&mut self, new_root: Expression) -> Expression
Replaces the root node with new_root, returning the old root node.
Sourcepub fn constraints(&self) -> &Vec<Expression>
pub fn constraints(&self) -> &Vec<Expression>
The top-level constraints in this model.
Sourcepub fn constraints_mut(&mut self) -> &mut Vec<Expression>
pub fn constraints_mut(&mut self) -> &mut Vec<Expression>
The top-level constraints in this model as a mutable vector.
Sourcepub fn clauses_mut(&mut self) -> &mut Vec<CnfClause>
pub fn clauses_mut(&mut self) -> &mut Vec<CnfClause>
The cnf clauses in this model as a mutable vector.
Sourcepub fn replace_constraints(
&mut self,
new_constraints: Vec<Expression>,
) -> Vec<Expression>
pub fn replace_constraints( &mut self, new_constraints: Vec<Expression>, ) -> Vec<Expression>
Replaces the top-level constraints with new_constraints, returning the old ones.
Sourcepub fn replace_clauses(&mut self, new_clauses: Vec<CnfClause>) -> Vec<CnfClause>
pub fn replace_clauses(&mut self, new_clauses: Vec<CnfClause>) -> Vec<CnfClause>
Replaces the cnf clauses with new_clauses, returning the old ones.
Sourcepub fn add_constraint(&mut self, constraint: Expression)
pub fn add_constraint(&mut self, constraint: Expression)
Adds a top-level constraint.
Sourcepub fn add_clause(&mut self, clause: CnfClause)
pub fn add_clause(&mut self, clause: CnfClause)
Adds a cnf clause.
Sourcepub fn add_constraints(&mut self, constraints: Vec<Expression>)
pub fn add_constraints(&mut self, constraints: Vec<Expression>)
Adds top-level constraints.
Sourcepub fn add_clauses(&mut self, clauses: Vec<CnfClause>)
pub fn add_clauses(&mut self, clauses: Vec<CnfClause>)
Adds cnf clauses.
Sourcepub fn add_symbol(&mut self, decl: DeclarationPtr) -> Option<()>
pub fn add_symbol(&mut self, decl: DeclarationPtr) -> Option<()>
Adds a new symbol to the symbol table.
Sourcepub fn into_single_expression(self) -> Expression
pub fn into_single_expression(self) -> Expression
Converts the constraints in this model to a single expression suitable for use inside another expression tree.
Sourcepub fn collect_stable_id_mapping(&self) -> HashMap<ObjId, ObjId>
pub fn collect_stable_id_mapping(&self) -> HashMap<ObjId, ObjId>
Collects all ObjId values from the model using uniplate traversal.