Reference

Struct Reference 

pub struct Reference {
    pub ptr: DeclarationPtr,
    pub repr: Option<ReprRulePtr>,
}
A reference to a declaration (variable, parameter, etc.)

This is a thin wrapper around DeclarationPtr with two main purposes:

  1. Encapsulate the serde pragmas (e.g., serializing as IDs rather than full objects)
  2. Enable type-directed traversals of references via uniplate

Fields

ptr: DeclarationPtrrepr: Option<ReprRulePtr>

Implementations

Source

impl Reference

pub fn new(ptr: DeclarationPtr) -> Self

pub fn ptr(&self) -> &DeclarationPtr

pub fn into_ptr(self) -> DeclarationPtr

pub fn name(&self) -> MappedRwLockReadGuard<'_, Name>

pub fn id(&self) -> ObjId

pub fn domain(&self) -> Option<DomainPtr>

pub fn resolved_domain(&self) -> Option<Moo<GroundDomain>>

pub fn select_repr<R: ReprRule + ?Sized>( &mut self, ) -> Result<MappedRwLockReadGuard<'_, R::DeclLevel>, ReprSelectError>

Select the given representation for this reference, if it is currently unrepresented and the representation exists for the underlying variable.

Errors
Returns

State of the initialised representation

pub fn select_repr_via( &mut self, rule: ReprRulePtr, ) -> Result<MappedRwLockReadGuard<'_, dyn ReprStateStored>, ReprSelectError>

Same as Reference::select_repr, but type-erased

pub fn select_or_init_repr_via( &mut self, rule: ReprRulePtr, ) -> ReprGetOrInitResult<'_, dyn ReprStateStored, ReferenceReprError>

Same as Reference::select_or_init_repr, but type-erased

pub fn select_or_init_repr<R: ReprRule + ?Sized>( &mut self, ) -> ReprGetOrInitResult<'_, R::DeclLevel, ReferenceReprError>

Select the given representation for this reference, initialising it if necessary. Will fail if a different representation is already selected.

Errors
  • ReprSelectError if a different representation is already selected for this reference
  • [ReprInitError] | [ReprInstantiateError] if the representation could not be initialised
Returns

(state, symbols, constraints) where:

  • state is an instance of the given representation
  • symbols are new variables created by the representation
  • constraints are new top-level constraints created by the representation
pub fn update_or_init_repr<R: ReprRule + ?Sized>( &mut self, ) -> ReprGetOrInitResult<'_, R::DeclLevel, ReprError>

Select the given representation for this reference, initialising it if necessary. Will overwrite the existing selection.

Errors
  • [ReprInitError] | [ReprInstantiateError] if the representation could not be initialised
Returns

(state, symbols, constraints) where:

  • state is an instance of the given representation
  • symbols are new variables created by the representation
  • constraints are new top-level constraints created by the representation
pub fn update_or_init_repr_via( &mut self, rule: ReprRulePtr, ) -> ReprGetOrInitResult<'_, dyn ReprStateStored, ReprError>

Same as Reference::update_or_init_repr, but type-erased

pub fn get_repr( &self, ) -> Option<(ReprRulePtr, MappedRwLockReadGuard<'_, dyn ReprStateStored>)>

If this reference has a representation selected, return (rule, state) where

  • rule is a pointer to the representation rule
  • state is an instance of that representation
pub fn get_repr_as<R: ReprRule + ?Sized>( &self, ) -> Option<MappedRwLockReadGuard<'_, R::DeclLevel>>

If this reference has this specific representation selected, get its state as a concrete type

pub fn resolve_expression(&self) -> Option<Expression>

Returns the expression behind a value-letting reference, if this is one.

Prefer Reference::with_resolved_expression when the expression only needs to be inspected. Returning an owned value here necessarily clones collection-bearing expressions such as matrix literals.

pub fn with_resolved_expression<T>( &self, inspect: impl FnOnce(&Expression) -> T, ) -> Option<T>

Calls inspect with the expression behind a value-letting reference.

The declaration is kept read-locked for the duration of inspect, allowing callers to ask questions about large value lettings without cloning them. Quantified declarations may delegate their value to an expression generator; those are handled here as well.

pub fn resolve_constant(&self) -> Option<Literal>

Evaluates this reference to a literal if it resolves to a constant.

pub fn resolve_atomic(&self) -> Option<Atom>

Resolves this reference to an atomic expression, if possible.