pub struct SymbolTablePtr{ /* private fields */ }Implementations
Sourceimpl SymbolTablePtr
impl SymbolTablePtr
Sourcepub fn new() -> Self
pub fn new() -> Self
Create an empty new SymbolTable and return a shared pointer to it
Sourcepub fn with_parent(symbols: SymbolTablePtr) -> Self
pub fn with_parent(symbols: SymbolTablePtr) -> Self
Create an empty new SymbolTable with the given parent and return a shared pointer to it
Sourcepub fn read(&self) -> RwLockReadGuard<'_, SymbolTable>
pub fn read(&self) -> RwLockReadGuard<'_, SymbolTable>
Read the underlying symbol table. This will block the current thread until a read lock can be acquired.
WARNING
- If the current thread already holds a lock over this table, this may deadlock.
Sourcepub fn write(&self) -> RwLockWriteGuard<'_, SymbolTable>
pub fn write(&self) -> RwLockWriteGuard<'_, SymbolTable>
Mutate the underlying symbol table. This will block the current thread until an exclusive write lock can be acquired.
WARNING
- If the current thread already holds a lock over this table, this may deadlock.
- Trying to acquire any other lock until the write lock is released will cause a deadlock.
- This will mutate the underlying data, which may be shared between other
SymbolTablePtrs. Make sure that this is what you want.
To create a separate copy of the table, see SymbolTablePtr::detach.