pub struct PtrAsInner;Expand description
Serialises a shared pointer to some object x as a tuple (ID, X), where:
IDis the unique and immutable ID of the object. (See: HasId)Xis a full copy of the object’s value. (Seex’s implementation of the Serialize trait for details)
On de-serialisation, independent copies of the object with the same ID are created.
WARNING
De-serialisation makes no attempt to restore shared pointers.
Two pointers to the same value x will be de-serialised as two separate copies
of it, x' and x''. Mutating the value of x' will not change the value of x''.
After de-serialising an entire structure, it is the user’s responsibility to go through it and manually restore the shared pointers. See the de-serialisation code for [Model] for an example.